STANDARD = [
    "CREEP",
    "DFLOW",
    "DFLUX",
    "DISP",
    "DLOAD",
    "FILM",
    "FLOW",
    "FRIC",
    "FRIC_COEF",
    "GAPCON",
    "GAPELECTR",
    "HARDINI",
    "HETVAL",
    "MPC",
    "ORIENT",
    "RSURFU",
    "SDVINI",
    "SIGINI",
    "UAMP",
    "UANISOHYPER_INV",
    "UANISOHYPER_STRAIN",
    "UCORR",
    "UCREEPNETWORK",
    "UDAMAGEMF",
    "UDECURRENT",
    "UDEMPOTENTIAL",
    "UDMGINI",
    "UDSECURRENT",
    "UEL",
    "UELEMDRESP",
    "UELMAT",
    "UEPACTIVATIONFACET",
    "UEPACTIVATIONSETUP",
    "UEPACTIVATIONVOL",
    "UEXPAN",
    "UEXTERNALDB",
    "UFIELD",
    "UFLUID",
    "UFLUIDCONNECTORLOSS",
    "UFLUIDCONNECTORVALVE",
    "UFLUIDLEAKOFF",
    "UFLUIDPIPEFRICTION",
    "UGENS",
    "UHARD",
    "UHYPEL",
    "UHYPER",
    "UHYPER_STRETCH",
    "UINTER",
    "UMASFL",
    "UMAT",
    "UMATHT",
    "UMDFLUX",
    "UMDFLUXSETUP",
    "UMESHMOTION",
    "UMIXMODEFATIGUE",
    "UMOTION",
    "UMULLINS",
    "UPOREP",
    "UPRESS",
    "UPSD",
    "URDFIL",
    "USDFLD",
    "USUPERELASHARDMOD",
    "UTEMP",
    "UTRACLOAD",
    "UTRS",
    "UTRSNETWORK",
    "UVARM",
    "UWAVE",
    "UXFEMNONLOCALWEIGHT",
    "VOIDRI",
]

EXPLICIT = [
    "VDFLUX",
    "VDISP",
    "VDLOAD",
    "VEXTERNALDB",
    "VFABRIC",
    "VFRIC",
    "VFRIC_COEF",
    "VFRICTION",
    "VHETVAL",
    "VUAMP",
    "VUANISOHYPER_INV",
    "VUANISOHYPER_STRAIN",
    "VUCHARLENGTH",
    "VUCREEPNETWORK",
    "VUEL",
    "VUEOS",
    "VUEXPAN",
    "VUFIELD",
    "VUFLUIDEXCH",
    "VUFLUIDEXCHEFFAREA",
    "VUHARD",
    "VUINTER",
    "VUINTERACTION",
    "VUMAT",
    "VUMATHT",
    "VUMULLINS",
    "VUSDFLD",
    "VUSUPERELASHARDMOD",
    "VUTRS",
    "VUVISCOSITY",
    "VWAVE",
]
subs = STANDARD + EXPLICIT
subs += [sub.lower() for sub in subs]
